<?PHP

// set include path
$inc_path = ini_get('include_path');
$inc_path .= PATH_SEPARATOR . dirname(__FILE__);
if (version_compare(PHP_VERSION, '5', '>='))
    $inc_path .= PATH_SEPARATOR . dirname(__FILE__) . '/php5';
else
    $inc_path .= PATH_SEPARATOR . dirname(__FILE__) . '/php4';
ini_set('include_path', $inc_path);

require_once 'lib/classes/Config.class.php';

// require domxml
if (version_compare(PHP_VERSION, '5', '>='))
    require_once 'vendor/php4-to-php5/domxml-php4-to-php5.php';

// require flexi templates
require_once 'vendor/flexi/flexi.php';

// require Creole
require_once 'creole/Creole.php';

require_once 'vips.config.php';
require_once 'vips_common.inc.php';

class VipsPlugin extends AbstractStudIPStandardPlugin
{
    var $active_tests;
    var $active_exams;

    function VipsPlugin ()
    {
        global $g_outlet, $vips_db;

        parent::AbstractStudIPStandardPlugin();

        if (version_compare(PHP_VERSION, '5', '>='))
            $options = 0;
        else
            $options = Creole::NO_ASSOC_LOWER();

        $g_outlet = enter();
        $vips_db = Creole::getConnection(array(
                                'phptype'  => 'mysql',
                                'hostspec' => $g_outlet->db_host,
                                'username' => $g_outlet->db_user,
                                'password' => $g_outlet->db_password,
                                'database' => $g_outlet->db_database
                        ), $options);

        // this plugin wants its own icon
        $this->setPluginiconname("images/vips_1.gif");
        $this->setChangeIndicatorIconName('images/vipsr_1.gif');

        // navigation
        $navigation =& new PluginNavigation();
        $navigation->setDisplayname(_('bungsblatt'));
        $navigation->addLinkParam('action', 'sheets');

        $admin_nav =& new PluginNavigation();
        $admin_nav->setDisplayname(_('Klausur'));
        $admin_nav->addLinkParam('action', 'exams');
        $navigation->addSubmenu($admin_nav);

        $admin_nav =& new PluginNavigation();
        $admin_nav->setDisplayname(_('Gruppenverwaltung'));
        $admin_nav->addLinkParam('action', 'groups');
        $navigation->addSubmenu($admin_nav);

        $admin_nav =& new PluginNavigation();
        $admin_nav->setDisplayname(_('Ergebnisse'));
        $admin_nav->addLinkParam('action', 'solutions');
        $navigation->addSubmenu($admin_nav);

        if ($g_outlet->status != 'student' && $g_outlet->status != 'nobody')
        {
            $admin_nav =& new PluginNavigation();
            $admin_nav->setDisplayname(_('Einstellungen'));
            $admin_nav->addLinkParam('action', 'admin');
            $navigation->addSubmenu($admin_nav);

            $admin_nav =& new PluginNavigation();
            $admin_nav->setDisplayname(_('Import'));
            $admin_nav->addLinkParam('action', 'import');
            $navigation->addSubmenu($admin_nav);
        }

        $admin_nav =& new PluginNavigation();
        $admin_nav->setDisplayname(_('Druckansicht'));
        $admin_nav->addLinkParam('action', 'export');
        $navigation->addSubmenu($admin_nav);

        $this->setNavigation($navigation);
        $this->setShownInOverview(TRUE);
    }

    function initialize ()
    {
        global $_include_additional_header;
        global $vipsPlugin, $vipsTemplateFactory;
        global $HELP_KEYWORD;

        $_include_additional_header =
            '<link rel="stylesheet" href="'.$this->getPluginPath().'/css/vips_style.css.php" type="text/css">';

        $vipsPlugin = $this;
        $vipsTemplateFactory = new Flexi_TemplateFactory(dirname(__FILE__).'/templates/');

        $HELP_TOPICS = array(
            'sheets'    => 'Vips.bungklausur',
            'exams'     => 'Vips.bungklausur',
            'groups'    => 'Vips.bungsgruppen',
            'solutions' => 'Vips.Lsung',
            'import'    => 'Vips.Aufgaben',
            // 'export' => 'Vips.Drucken',      // TODO
            'admin'     => 'Vips.Vipad');

        $action = getExternalParam('action');
        $HELP_KEYWORD = $HELP_TOPICS[$action];

        if (!isset($HELP_KEYWORD))
            $HELP_KEYWORD = 'Vips';
    }

    function getPluginname ()
    {
        return _('Vips');
    }

    function getDisplaytitle ()
    {
        return $this->getPluginname();
    }

    function actionShow ()
    {
        global $g_outlet;

        $error_level = error_reporting(E_ALL & ~E_NOTICE & ~E_WARNING);
        $display_errors = ini_set('display_errors', true);

        // action: sheets, exams, groups, solution, admin, import or export
        $action = getExternalParam('action');
        // subaction: e.g. store_exercise, delete_exam and the like
        $subaction = getExternalParam('subaction');

        if ($g_outlet->status == 'nobody')
        {
            StudIPTemplateEngine::showErrorMessage(_('Ihre Session ist leider abgelaufen. Bitte loggen Sie sich erneut ein.'));
            return;
        }

        switch ($action)
        {
            case 'sheets':
                require_once 'vips_assignments.inc.php';
                vips_assignments($action, $subaction);
                break;

            case 'exams':
                require_once 'vips_assignments.inc.php';
                vips_assignments($action, $subaction);
                break;

            case 'groups':
                require_once 'vips_groups.inc.php';
                vips_groups($subaction);
                break;

            case 'solutions':
                require_once 'vips_solutions.inc.php';
                vips_solutions($subaction);
                break;

            case 'import':
                require_once 'vips_import.inc.php';
                vips_import($subaction);
                break;

            case 'export':
                require_once 'vips_export.inc.php';
                vips_export($subaction);
                break;

            case 'admin':
                require_once 'vips_admin.inc.php';
                vips_admin($subaction);
                break;

            default:
                require_once 'vips_assignments.inc.php';
                vips_assignments('sheets', 'list_assignments');
        }

        error_reporting($error_level);
        ini_set('display_errors', $display_errors);
    }

    function actionShowConfigurationPage ()
    {
        global $g_outlet, $vipsTemplateFactory;

        $user = $this->getUser();
        $permission = $user->getPermission();

        if (!$permission->hasRootPermission())
        {
            StudIPTemplateEngine::showErrorMessage(_('Sie besitzen keine Berechtigung, um dieses Plugin zu konfigurieren.'));
            return;
        }

        $config = new Config();
        $stud_id = getExternalParam('stud_id');

        if ($stud_id == '')
        {
            $stud_id = $config->getValue('VIPS_STUDENT_ID_DATAFIELD');
        }
        else
        {
            $config->setValue($stud_id, 'VIPS_STUDENT_ID_DATAFIELD');
        }

        $template = $vipsTemplateFactory->open('config_main');
        $template->set_attribute('stud_id', $stud_id);
        $template->set_attribute('buttonpath', $g_outlet->buttonpath);

        echo $template->render();
    }

    function getPluginExistingItems ($range_id)
    {
        return 0;
    }

    function getOverviewMessage ($has_changed)
    {
        if ($this->active_tests + $this->active_exams == 0)
            return $this->getPluginname();

        if ($this->active_tests > 0)
            $active_items[] = $this->active_tests.' '.
                                ngettext('neue bung', 'neue bungen', $this->active_tests);
        if ($this->active_exams > 0)
            $active_items[] = $this->active_exams.' '.
                                ngettext('neue Klausur', 'neue Klausuren', $this->active_exams);

        return join(', ', $active_items);
    }

    function hasChanged ($lastlogin)
    {
        global $perm, $vips_db;

        $seminar = $this->getId();
        $user = $this->getUser();
        $user_id = $user->getUserid();
        $date = date("Y-m-d H:i:s");

        // don't highlight icon for tutors and lecturers
        // NOTE: Permission::hasTutorPermissionInPOI() needs to be fixed
        if ($perm->have_studip_perm('tutor', $seminar))
            return false;

        // find all active tests not yet seen by the student
        $result = $vips_db->executeQuery('SELECT vips_uebungsblatt.ID FROM vips_uebungsblatt
                    LEFT JOIN vips_aufgaben_zeit
                    ON vips_uebungsblatt.ID = vips_aufgaben_zeit.vips_uebungsblatt AND
                        vips_aufgaben_zeit.user_id = \''.$user_id.'\'
                    WHERE vips_uebungsblatt.Kurs = \''.$seminar.'\' AND
                        vips_uebungsblatt.Beginn <= \''.$date.'\' AND
                        vips_uebungsblatt.Ende > \''.$date.'\' AND
                        vips_aufgaben_zeit.user_id IS NULL');
        $this->active_tests = $result->getRecordCount();

        // find all active exams not yet seen by the student
        $result = $vips_db->executeQuery('SELECT vips_klausur.ID FROM vips_klausur
                    LEFT JOIN vips_aufgaben_zeit
                    ON vips_klausur.ID = vips_aufgaben_zeit.vips_klausur AND
                        vips_aufgaben_zeit.user_id = \''.$user_id.'\'
                    WHERE vips_klausur.Kurs = \''.$seminar.'\' AND
                        vips_klausur.Beginn <= \''.$date.'\' AND
                        vips_klausur.Ende > \''.$date.'\' AND
                        vips_aufgaben_zeit.user_id IS NULL');
        $this->active_exams = $result->getRecordCount();

        return $this->active_tests + $this->active_exams > 0;
    }
}

?>
