<?php

/* studip/vips/exercises/rh_exercise.php
 * 
 * Reihenfolge-Aufgabe. Eine Liste von Texten, die einander zugeordnet werden mssen.
 */

require_once 'exercises/exercise.php';

class rh_exercise extends Exercise
{
        
        var $g_numberOfAnswers = 6;
        var $g_exerciseName = "Reihenfolge";
        var $g_exerciseDescription = "Texte m&uuml;ssen in die richtige Reihenfolge gebracht werden";
        
        
        
        /* Constructor */
        function rh_exercise()
        {
                $this->g_Action = getExternalParam('action');
                $this->g_OpenObject = 'openobject_'.strtolower($this->g_Action);
        }
        
        
        
        /**
         * Computes points.
         */
        function evaluate($exercise, $solution)
        {
                $points = 0;
                
                $dom_xml = $this->getDomXML($exercise);
                $answers = $dom_xml->get_elements_by_tagname("Answer");
                
                $answer_array = array();
                $correct_array = array();
                
                foreach ($answers as $node)
                {
                        $answer_array[]         = utf8_decode($node->get_content());
                        $correct_array[]        = $node->get_attribute("Default");
                }
                
                $student_array = $this->getTagsFromXML($solution, "answer");
                
                for ($i = 0; $i < count($correct_array); $i++)
                {
                        // if correct answer was given, give a point
                        if ($correct_array[$i] == $student_array[$i])
                        {
                                $points++;
                        }
                        
                        // don't subtract points if an answer was wrong
                }
                 
                $m_safe = 0;
                if ($points <= count($answer_array) && $points >= 0)
                {
                        $m_safe = 1;
                }

                $m_percent = $points * 100 / count($correct_array);
                
                $m_return[1] = $m_percent;
                $m_return[2] = $m_safe;
                $m_return[5] = $points;
                
        return $m_return;
        }
        
        
        
        /**
         * Creates XML to store student's solution.
         */
        function genSolution($p_exercise_id, $p_number_of_answers)
        {
                $i = 0;
                $xml = "<solution id=\"".$p_exercise_id."\">";
                
                foreach (getExternalParam("student_answer") as $answer)
                {
                        $xml .= "<answer id=\"".$i++."\">";
                        $xml .= "<body>$answer</body>";
                        $xml .= "</answer>";
                }
                
                $xml .= "</solution>";
                
                return $xml;
        }
        
        
        
        /**
         * Returns XML data representing the exercise, ready to be stored in the
         * database.
         * 
         * @returns String The XML code representing the exercise.
         */
        function getExerciseConstruct()
        {
                $exercise_language = "de";
                
                /* allgemeine Daten */
                $exercise_type     = getExternalParam("exercise_type");
                $exercise_name     = getExternalParam("exercise_name");
                $exercise_question = getExternalParam("exercise_question");
                $exercise_hint     = getExternalParam("exercise_hint");
                $exercise_comment  = getExternalParam("exercise_comment");
                $exercise_pic_url  = getExternalParam("exercise_pic_url");  //??
                $provides          = getExternalParam("provides");  //??
                $depends           = getExternalParam("depends"); //??
                $optional_depends  = getExternalParam("optional_depends"); //??
                $number_of_answers = getExternalParam("number_of_answers");
                
                /* spezielle Daten */
                $default_array = array();
                for ($i = 0; $i < $number_of_answers; $i++)
                {
                        $tmp_default = getExternalParam("default_$i");
                        $default_array[$i] = $tmp_default;
                }
                
                $answer_array = array();
                $default_to_answer_array = array();
                for ($i = 0; $i < $number_of_answers; $i++)
                {
                        $tmp_answer = getExternalParam("answer_$i");
                        $answer_array[$i] = $tmp_answer;
                        $default_to_answer_array[$i] = getExternalParam("default_to_answer_$i");
                }
                
                $xml = "<TestItem ID=\"$exercise_type\" Type=\"Reihenfolge\" Language=\"$exercise_language\">";
                $xml .= "<Name><![CDATA[".$exercise_name."]]></Name>";
                $xml .= "<Hint><![CDATA[".$exercise_hint."]]></Hint>";
                $xml .= "<CommentField Visible=\"".$exercise_comment."\"></CommentField>";
                $xml .= "<PictureURL><![CDATA[".$exercise_pic_url."]]></PictureURL>";
        $xml .= "<Provides><![CDATA[".$provides."]]></Provides>";
        $xml .= "<Depends><![CDATA[".$depends."]]></Depends>";
        $xml .= "<OptionalDepends><![CDATA[".$optional_depends."]]></OptionalDepends>";
                $xml .= "<Question>";
                $xml .= "<Paragraph><![CDATA[".$exercise_question."]]></Paragraph>";
                $xml .= "</Question>";
                for ($i = 0; $i < count($default_array); $i++)
                {
                        $xml .= "<Default ID=\"$i\"><![CDATA[".$default_array[$i]."]]></Default>";
                }
                for ($i = 0; $i < count($answer_array); $i++)
                {
                        $xml .= "<Answer ID=\"$i\" Default=\"".$default_to_answer_array[$i]."\"><![CDATA[".$answer_array[$i]."]]></Answer>";
                }
                $xml .= "</TestItem>";
                
                return $xml;
                
                
                /*** TEST ***/
/*              $doc = domxml_new_doc("1.0");
                $root = $doc->create_element("TestItem");
                $doc->append_child($root);
                $name = $doc->create_element("Name");
                $root->append_child($name);
                $content = $doc->create_cdata_section($exercise_name);
                $name->append_child($content);
                return $doc->dump_mem();
*/      }
        
        
        
        /**
         * Moves an answer up or down.
         */
        function move_answer($p_xml, $p_position, $p_direction)
        {
                $dom_doc = $this->getDomXML($p_xml);
                $answers = $dom_doc->get_elements_by_tagname("Answer");
                
                $node = $answers[$p_position];
                
                if ($p_direction == "up")
                {
                        $prev_node = $node->previous_sibling();
                        $prev_node->set_attribute("ID", $prev_node->get_attribute("ID") + 1);
                        $node->set_attribute("ID", $node->get_attribute("ID") - 1);
                        $node->insert_before($node, $prev_node);
                }
                else if ($p_direction == "down")
                {
                        $next_node = $node->next_sibling();
                        $next_node->set_attribute("ID", $next_node->get_attribute("ID") - 1);
                        $node->set_attribute("ID", $node->get_attribute("ID") + 1);
                        $node->insert_before($next_node, $node);
                }
                
                return utf8_decode($dom_doc->dump_mem());
        }
        
        
        
        /**
         * Adds a given number of empty pairs to XML.
         */
        function add_pairs($p_xml, $p_count)
        {
                $dom_doc = $this->getDomXML($p_xml);
                $defaults = $dom_doc->get_elements_by_tagname("Default");
                $answers = $dom_doc->get_elements_by_tagname("Answer");
                
                if (count($defaults) != count($answers))
                {
                        // Something went wrong, as number of defaults is inequal to number of answers!
                        return false;
                }
                
                $number_of_pairs = count($defaults);
                $root = $dom_doc->document_element();
                
                for ($i = 0; $i < $p_count; $i++)
                {
                        $empty_default_node = $dom_doc->create_element("Default");
                        $empty_answer_node = $dom_doc->create_element("Answer");
                        
                        $empty_default_node->append_child($dom_doc->create_attribute("ID", $i+$number_of_pairs));
                        $empty_default_node->append_child($dom_doc->create_cdata_section(""));
                        
                        $empty_answer_node->append_child($dom_doc->create_attribute("ID", $i+$number_of_pairs));
                        $empty_answer_node->append_child($dom_doc->create_attribute("Default", $i+$number_of_pairs));
                        $empty_answer_node->append_child($dom_doc->create_cdata_section(""));
                        
                        $root->append_child($empty_default_node);
                        $root->append_child($empty_answer_node);
                }
                
                return utf8_decode($dom_doc->dump_mem());
        }
        
        
        
        /**
         * Randomizes order of answers.
         */
        function mix_answers($p_xml)
        {
                $dom_doc = $this->getDomXML($p_xml);
                $answers = $dom_doc->get_elements_by_tagname("Answer");
                
                $i = 0;
                foreach ($answers as $node)
                {
                        $ids[] = $i++;
                }
                srand((float)microtime() * 1000000);
                shuffle($ids);
                
                foreach ($answers as $node)
                {
                        $node->set_attribute("ID", array_shift($ids));
                }
                
                $answers = $this->sort_by_id($answers);
                
                return utf8_decode($dom_doc->dump_mem());
        }
        
        
        
        /**
         * Deletes pairs from XML.
         * 
         * If you do not set <tt>$p_id</tt>, all unassigned pairs are deleted,
         * otherwise the <tt>Default</tt> with the id <tt>$p_id</tt> and the
         * corresponding <tt>Answer</tt> are removed.
         */
        function delete_pairs($p_xml, $p_id = false)
        {
                $dom_doc = $this->getDomXML($p_xml);
                $defaults = $dom_doc->get_elements_by_tagname("Default");
                $answers = $dom_doc->get_elements_by_tagname("Answer");
                
                if (count($defaults) != count($answers))
                {
                        // Something went wrong, as number of defaults is inequal to number of answers!
                        return false;
                }
                
                
                
                // delete only one pair
                if ($p_id !== false)
                {
                        $defaults[$p_id]->unlink_node();
                        
                        foreach ($answers as $answer_node)
                        {
                                if ($answer_node->get_attribute("Default") == $p_id)
                                {
                                        $answer_node->unlink_node();
                                        break;
                                }
                        }
                }
                
                // delete all empty pairs
                else
                {
                        // check each default node
                        foreach ($defaults as $default_node)
                        {
                                $id = $default_node->get_attribute("ID");
                                
                                // Is default node empty?
                                if (isset($default_node) && $default_node->get_content() == "")
                                {
                                        // look for the corresponding answer node
                                        foreach ($answers as $answer_node)
                                        {
                                                if (isset($answer_node) && $answer_node->get_attribute("Default") == $id)
                                                {
                                                        // Corresponding node found... Is it empty?
                                                        if ($answer_node->get_content() == "")
                                                        {
                                                                // yes, it is empty...
                                                                $default_node->unlink_node();
                                                                $answer_node->unlink_node();
                                                        }
                                                }
                                        }
                                }
                        }
                }
                
                // keep ids in order and without "holes": 0, 1, 2, ..., n
                $this->reassign_ids
                                ( $dom_doc->get_elements_by_tagname("Default")
                                , $dom_doc->get_elements_by_tagname("Answer")
                                );
                
                return utf8_decode($dom_doc->dump_mem());
        }
        
        
        
        /**
         * Checks a student's solution whether two or more answers are assigned to
         * one single default.
         */
        function checkSolutionXML($p_xml)
        {
                if (!isset($p_xml)) return 0;   // FIXME is this correct?

                $dom_doc = $this->getDomXML($p_xml);
                $answers = $dom_doc->get_elements_by_tagname("answer");
                $already_assigned = array();
                
                foreach ($answers as $answer)
                {
                        $content = $answer->get_content();
                        
                        if ($content == "NULL") continue;  // skip NULL values
                        
                        // double entry found!
                        if (array_search($content, $already_assigned) !== false)
                        {
                                $double_assigned = true;
                        }
                        array_push($already_assigned, $content);
                }
                
                if ($double_assigned) return 0;
                else return 1;
        }
        
        
        
        function sort_by_id($p_array)
        {
                // "Bubble Sort" :-)
                $i = count($p_array)-1;
                $first_node = $p_array[0];
                while ($i >= 0)
                {
                        foreach ($p_array as $node)
                        {
                                // look for greatest id and put it to the front
                                if ($node->get_attribute("ID") == $i)
                                {
                                        if ($node != $first_node)
                                        {
                                                $node->insert_before($node, $first_node);
                                        }
                                        $first_node = $node;
                                        $i--;
                                }
                        }
                }
                return $p_array;
        }
        
        
        
        function reassign_ids($p_defaults, $p_answers)
        {
                // for each default
                for ($i = 0; $i < count($p_defaults); $i++)
                {
                        // is id not in order?
                        if (($old_id = $p_defaults[$i]->get_attribute("ID")) != $i)
                        {
                                // correct the id
                                $p_defaults[$i]->set_attribute("ID", $i);
                                
                                // find the corresponding answer
                                foreach ($p_answers as $answer_node)
                                {
                                        if ($answer_node->get_attribute("Default") == $old_id)
                                        {
                                                // update answer's "Default" attribute
                                                $answer_node->set_attribute("Default", $i);
                                        }
                                }
                        }
                        
                        // set correct id for answer
                        $p_answers[$i]->set_attribute("ID", $i);
                }
        }
} // end class
