<?php
/*
 *  $Id: PrimaryKeyInfo.php,v 1.4 2005/03/01 13:56:43 micha Exp $
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many individuals
 * and is licensed under the LGPL. For more information please see
 * <http://creole.phpdb.org>.
 */

/**
 * Represents a PrimaryKey
 *
 * @author    Hans Lellelid <hans@xmpl.org>
 * @author    Michael Aichler <aichler@mediacluster.de>
 * @version   $Revision: 1.4 $
 * @package   creole.metadata
 */
class PrimaryKeyInfo
{
  /**
  * Name of the primary key
  * @var string
  */
  var $name;
  /**
  * Columns in the primary key
  * @var array
  */
  var $columns = array();
  /**
  * Additional vendor specific information
  * @var array
  */
  var $vendorSpecificInfo = array();


  /**
  * PrimaryKeyInfo constructor.
  *
  * @param string $name The name of the foreign key.
  * @param array $vendorInfo Vendor specific info array.
  */
  function PrimaryKeyInfo($name, $vendorInfo = array())
  {
    $this->name = $name;
    $this->vendorSpecificInfo = $vendorInfo;
  }

  /**
  * Get foreign key name.
  *
  * @return string
  */
  function getName()
  {
      return $this->name;
  }

  /**
  * Returns an array of columns which are in the primary key.
  *
  * @return array
  */
  function & getColumns()
  {
    return $this->columns;
  }

  /**
  * Get vendor specific optional information for this primary key.
  *
  * @return array vendorSpecificInfo[]
  */
  function getVendorSpecificInfo()
  {
    return $this->vendorSpecificInfo;
  }

  /**
  * Add a column which is in the primary key.
  *
  * @param ColumnInfo $column
  * @return void
  */
  function addColumn(&$column)
  {
    Creole::typeHint($column, 'ColumnInfo', 'PrimaryKeyInfo', 'addColumn', 1);
    $this->columns[] =& $column;
  }

  /**
  * @return string
  */
  function toString()
  {
    return $this->name;
  }
  
}
