<?PHP

/**
 * This file contains classes and functions needed everywhere in the
 * vips code.
 * Several functions here were from former versions of vips but aren't used anymore but
 * I left them in here, might help...
 * @author: Phillip Huegelmeyer, Christa Deiwiks
 * @version: 2006-08-11
 */

/**
 * Originally created to do more than htmlspecialchars()...
 * FIXME should use htmlReady() from visual.inc.php
 * 
 * @author:cdeiwiks
 */
function html_ready ($string) {
        $string = htmlspecialchars($string);
        $string = preg_replace('/&amp;#(x[0-9a-f]+|[0-9]+);/i', '&#$1;', $string);
        return $string;
}

/**
 * Return URL of Vips dispatcher script.
 * $params can contain optional additional parameters
 */
function vips_url ($params) {
    global $vipsPlugin;

    return PluginEngine::getLink($vipsPlugin, $params);
}

/**
 * Return URL of Vips dispatcher script for action
 * $params can contain optional additional parameters
 */
function vips_cmd ($cmd, $params) {
    global $vipsPlugin;

    return PluginEngine::getLink($vipsPlugin, $params, $cmd);
}

/**
 * 
 * Cuts a double after specified kommastellen, e.g.: 
 * roundLikeFloor(2.6318,3) => 2.631
 * roundLikeFloor(2.6315,0) => 2
 * 
 * @param: double $double Double to round
 * @param: int $kommastelle Number of kommastellen to be retained
 * @return: double $double The cut-off double
 * @author: cdeiwiks
 */
function roundLikeFloor($double, $kommastelle) {
        $factor = pow(10,$kommastelle);
        $double = $double*$factor;
        $double = floor($double);
        $double = $double/$factor;
        return $double;
}

/**
 * Returns a normalized version of a string (see comments within function)
 * NOTE: This function is _completely_ broken...
 * 
 * @param: String $string String to be normalized
 * @return: The normalized string
 * @author: cdeiwiks
 */
function normalizeText($string) {       
        
        ################################
        # line break -> one blank      #
        # multiple blanks -> one blank #
        ################################
        $string = ereg_replace("/[\r\n\w+]/",' ', $string);
        
        #####################################################
        # delete blanks before and after ()[]{}:+-/=,;.!?'" #
        #####################################################
        // FIXME nice try, but this is all wrong...
        $string = ereg_replace("/\w+\(\w+/",'(', $string);
        $string = ereg_replace("/\w+\)\w+/",')', $string);
        $string = ereg_replace("/\w+\[\w+/",'[', $string);
        $string = ereg_replace("/\w+\]\w+/",']', $string);
        $string = ereg_replace("/\w+\{\w+/",'{', $string);
        $string = ereg_replace("/\w+\}\w+/",'}', $string);
        $string = ereg_replace("/\w+\:\w+/",':', $string);
        $string = ereg_replace("/\w+\+\w+/",'+', $string);
        $string = ereg_replace("/\w+\-\w+/",'-', $string);
        $string = ereg_replace("/\w+\/\w+/",'/', $string);
        $string = ereg_replace("/\w+\=\w+/",'=', $string);
        $string = ereg_replace("/\w+\,\w+/",',', $string);
        $string = ereg_replace("/\w+\;\w+/",';', $string);
        $string = ereg_replace("/\w+\.\w+/",'.', $string);
        $string = ereg_replace("/\w+\!\w+/",'!', $string);
        $string = ereg_replace("/\w+\?\w+/",'?', $string);
        $string = ereg_replace("/\w+\'\w+/","'", $string);
        $string = ereg_replace("/\w+\"\w+/",'"', $string);
        ###########################
        # all chars to lower case #
        ###########################
        $string = strtolower($string);
        return $string;
}

/**
* Returns the value of a given variable. Looks up
* the POST and then the GET environment.
*
* If not found a given default value will be used.
*
* Uses trim(return_value).
*
* @param String $p_varName the Name of the variable
* @param String $p_defaultValue the default value to use if variable is not
*        found
* @return String $m_varValue the value of the variable.
*            Empty if nothing will be found.
*
* @author ??
* @version 2003-11-17
*/

function getExternalParam($p_varName, $p_defaultValue = "") {

        global $_VIPS;

        $m_varValue = $_POST[$p_varName];

        if($m_varValue == NULL || $m_varValue == "") {

                $m_varValue = $_GET[$p_varName];
                //vv not needed anymore //
                if($m_varValue == NULL || $m_varValue == "") {

                        $m_varValue = $_VIPS[$p_varName];

                        if($m_varValue == NULL || $m_varValue == "") {
                                $m_varValue = $p_defaultValue;
                        }
                }//==
        }
        if (!is_array($m_varValue)) { $m_varValue = trim($m_varValue); }
        // the debug line below can be switched of by commenting it.
        // echo '<table cellpadding="0" cellspacing="0" width="300" style="font-size:12px;background-color:lightgrey;"><tr><td width="150">'.$p_varName.'</td><td>:'.$m_varValue.'</td></tr></table>';
        //vv     for FH?      vv//
        /*if (!get_magic_quotes_gpc())
           $m_varValue = mysql_real_escape_string($m_varValue);*/ //or addslashes?
        return $m_varValue;
} // end function

/**
 * Function returns the maximum number of points a person or a group
 * can get for a given exercise sheet.
 *
 * @param integer the id of the exercise sheet
 * @param string the type of the exercise sheet
 * @return integer the number of maximal points for this exercise sheet
 * @author ?
 */

function get_max_points_assignment($p_exercise_sheet_id, $p_exercise_sheet_type) {
        global $vips_db;

        switch ($p_exercise_sheet_type) {
            case "vips_klausur":
                $m_insheetType = "vips_inKlausur";
                break;
            case "vips_uebungsblatt":
                $m_insheetType = "vips_inUebungsblatt";
                break;
            case "vips_gruppe":
                $m_insheetType = "vips_inGruppe";
                break;
        }
        $query = "SELECT SUM(Punkte) FROM ".$m_insheetType." WHERE ".$p_exercise_sheet_type."='".$p_exercise_sheet_id."'";
        $result = $vips_db->executeQuery($query);
        $result->next();
        $m_points = $result->get("SUM(Punkte)");
        if($m_points == "") {
                $m_points = 0;
        }
        return($m_points);
}

/**
 * NOT USED
 * Function returns the reached number of points of a person or a group
 * for a given exercise sheet.
 *
 * @param integer the id of the exercise sheet
 * @param string the type of the exercise sheet
 * @return integer the number of maximal points for this exercise sheet
 */
/*
function get_reached_points_assignment($p_exercise_sheet_id, $p_exercise_sheet_type, $p_userOrgroupID) {
        global $vips_db;

        if ($p_exercise_sheet_type == "vips_klausur")
        {
                // this query is correct but *VERY*(!!!) slow
//              $query = "SELECT SUM(`vips_K-Loesung-Punkte`.Punkte) AS ReachedPoints
//                      FROM `vips_K-Loesung`
//                              , `vips_K-Loesung-Punkte`
//                              , `vips_inKlausur`
//                      WHERE `vips_K-Loesung`.Loesungsid = `vips_K-Loesung-Punkte`.idLoesung
//                              AND `vips_K-Loesung`.Aufgabenid = `vips_inKlausur`.Aufgabe
//                              AND `vips_K-Loesung`.Klausurid = '$p_exercise_sheet_id'
//                              AND `vips_K-Loesung`.Userid = '$p_userOrgroupID'
//                              AND `vips_K-Loesung`.isLastSolution = 't'
//                      ";

                // this query is almost always correct (but counts also exercises which WERE in an
                // exercise sheet, were solved by students and have been removed afterwards) and
                // much quicker

                #$query = "SELECT DISTINCT SUM(`vips_K-Loesung-Punkte`.Punkte) AS ReachedPoints
                #       FROM `vips_K-Loesung`
                #               , `vips_K-Loesung-Punkte`
                #       WHERE `vips_K-Loesung`.Loesungsid = `vips_K-Loesung-Punkte`.idLoesung
                #               AND `vips_K-Loesung`.Klausurid = '$p_exercise_sheet_id'
                #               AND `vips_K-Loesung`.Userid = '$p_userOrgroupID'
                #               AND `vips_K-Loesung`.isLastSolution = 't'
                #       ";
                $query = "SELECT DISTINCT Aufgabenid, `vips_K-Loesung-Punkte`.Punkte "
                        ."FROM `vips_K-Loesung` "
                        ."    , `vips_K-Loesung-Punkte` "
                        ."    , `vips_inKlausur` "
                        ."WHERE `vips_K-Loesung`.Loesungsid = `vips_K-Loesung-Punkte`.idLoesung "
                        ."    AND `vips_K-Loesung`.Aufgabenid = `vips_inKlausur`.vips_aufgabe "
                        ."    AND `vips_K-Loesung`.Klausurid = '".$p_exercise_sheet_id."' "
                        ."    AND `vips_K-Loesung`.Userid = '".$p_userOrgroupID."' "
                        ."    AND `vips_K-Loesung`.isLastSolution = 't' "
                        ;
                $result = $vips_db->executeQuery($query);

                // Sorry, but MySQL cannot compute a SUM in connection with DISTINCT.
                while ($result->next())
                {
                        $m_reached_points += $result->get("Punkte");
                }

                return $m_reached_points;
        }
        else
        {
//              $query = "SELECT SUM(`vips_T-Loesung-Punkte`.Punkte) AS ReachedPoints
//                      FROM `vips_T-Loesung`
//                              , `vips_T-Loesung-Punkte`
//                              , `inUebungsblatt`
//                      WHERE `vips_T-Loesung`.Uebungsblattid = '$p_exercise_sheet_id'
//                              AND `vips_T-Loesung`.Gruppenid = '$p_userOrgroupID'
//                              AND `vips_T-Loesung`.isLastSolution = 't'
//                              AND `vips_T-Loesung`.Aufgabenid = `inUebungsblatt`.Aufgabe
//                              AND `vips_T-Loesung`.Loesungsid = `vips_T-Loesung-Punkte`.idLoesung
//                      ";
//
                $query = "SELECT SUM(`vips_T-Loesung-Punkte`.Punkte) AS ReachedPoints
                        FROM `vips_T-Loesung`
                                , `vips_T-Loesung-Punkte`
                        WHERE `vips_T-Loesung`.Loesungsid = `vips_T-Loesung-Punkte`.idLoesung
                                AND `vips_T-Loesung`.Uebungsblattid = '$p_exercise_sheet_id'
                                AND `vips_T-Loesung`.Gruppenid = '$p_userOrgroupID'
                                AND `vips_T-Loesung`.isLastSolution = 't'
                        ";
        }
        $result = $vips_db->executeQuery($query);
        $result->next();
        return($result->get("ReachedPoints"));
}
*/

?>
