<?php

// error_reporting(E_ALL);
// ini_set('display_errors', true);

require_once 'vips_assignments.inc.php';
require_once 'lib/classes/Config.class.php';
require_once 'lib/classes/DataFields.class.php';
// DataFieldEntry is not yet available
// require_once 'lib/classes/DataFieldEntry.class.php';

/**
 * 
 * Handles export of assignments, print preview etc (so far)
 * 
 * @author: Christa Deiwiks <cdeiwiks@uos.de>
 */
function vips_export($p_subaction) {
        if ($p_subaction == 'main')   { 
                main();
        }
        elseif ($p_subaction == 'print_overview') {
                print_overview();
        }
        elseif ($p_subaction == 'print_student_overview') {
                print_student_overview();
        }
        elseif ($p_subaction == 'print_assignment') {
                print_assignment();
        }
        elseif ($p_subaction == 'print_all_assignments') {
                print_all_assignments();
        }
        else print_overview();
}
 
function main() {
        global $g_outlet, $vipsTemplateFactory;

        $t_main = $vipsTemplateFactory->open('export_main');

        $t_main->set_attributes(array(
                'buttonpath'  => $g_outlet->buttonpath,
                'picturepath' => $g_outlet->picturepath,
                'status'      => $g_outlet->status));                                   

        echo $t_main->render();
}

/**
 * Lists all students of the course
 * => print solution of one/all student(s) for a given assignment
 *
 * get participants of course
 * list students with checkboxes to select one,two,all students
 * button:print assignment for selected students
 * new page for every student... pdf?
 */
function print_student_overview()
{
        global $vips_db, $g_outlet, $vipsTemplateFactory, $vipsPlugin;
        global $g_assignment_table, $g_inAssignment_table, $g_solution_table;
        
        $type   = getExternalParam("type");
        $mode   = getExternalParam("mode");
        $assignment     = getExternalParam("assignment");
        $print_correction_default      = getExternalParam("print_correction_default");
        $print_sample_solution_default = getExternalParam("print_sample_solution_default");
        
        if ($type == "exams") {
                $g_assignment_table     = 'vips_klausur';
                $g_inAssignment_table   = 'vips_inKlausur';
                $g_solution_table       = 'vips_K_Loesung';
        } else {
                $g_assignment_table     = 'vips_uebungsblatt';
                $g_inAssignment_table   = 'vips_inUebungsblatt';
                $g_solution_table       = 'vips_T_Loesung';
        }
        
        $m_course_id = $g_outlet->courseID;
        
        // fetch name of exam/exercise sheet
        $query = "SELECT Name FROM $g_assignment_table WHERE ID = $assignment";
        $result = $vips_db->executeQuery($query);
        $result->next();
        $name = $result->get("Name");
        
        /////////////////////////////////////////////////////
        // Fetch all exercises that are in the assignment. //
        /////////////////////////////////////////////////////
        
        $m_all_exercises = array();
        
        $query = "SELECT DISTINCT
                        $g_inAssignment_table.`vips_aufgabe`
                        , `vips_aufgabe`.Name
                        , $g_inAssignment_table.`Position`
                FROM
                        $g_inAssignment_table
                        , `vips_aufgabe`
                WHERE
                        $g_inAssignment_table.$g_assignment_table = '$assignment'
                        AND vips_aufgabe.ID = $g_inAssignment_table.`vips_aufgabe`
                ORDER BY
                        $g_inAssignment_table.`Position`
                ";
        $result = $vips_db->executeQuery($query);
        
        while ($result->next())
        {
                $temp_ex = array
                        ( "id" => $result->get("vips_aufgabe")
                        , "name" => $result->get("Name")
                        , "position" => $result->get("Position")
                        );
                array_push($m_all_exercises, $temp_ex);
        }
        
        //////////////////////////////////////////////////////////////
        // Fetch all participants in this course from the database. //
        //////////////////////////////////////////////////////////////
        
        $m_participants = array();
        
        if ($g_outlet->status != 'student')
        {
                $query = "SELECT DISTINCT
                                seminar_user.user_id
                        FROM
                                seminar_user,
                                auth_user_md5
                        WHERE
                                Seminar_id = '$m_course_id' AND
                                seminar_user.user_id = auth_user_md5.user_id
                        ORDER BY Nachname, Vorname";
                $result = $vips_db->executeQuery($query);
                
                while ($result->next())
                {
                        $m_user_id = $result->get("user_id");
                        
                        if (get_perm($m_course_id, $m_user_id) == "autor") { // skip tutors
                                $m_participants[] = array(
                                        "id"            => $m_user_id,
                                        "name"          => get_fullname($m_user_id),
                                        "username"      => get_username($m_user_id),
                                        "single_solver" => true);
                        }
                }
        }
        else
        {
                $m_participants[] = array(
                        "id"            => $g_outlet->userID,
                        "name"          => get_fullname($g_outlet->userID),
                        "username"      => get_username($g_outlet->userID),
                        "single_solver" => true);
        }
        
        $m_solvers = $m_participants;
        
        //////////////////////////////////////////////////////////////////////
        // Merge participants to groups if assignment is an exercise sheet. //
        //////////////////////////////////////////////////////////////////////
        # cdeiwiks: if no group, list as single participant
        
        if ($type == "sheets")
        {
                $m_solvers = array();
                $m_groups = array();
                
                foreach ($m_participants as $m_participant)
                {
                        $m_user_id = $m_participant["id"];
                        $fullname  = $m_participant["name"];
                        $username  = $m_participant["username"];
                        $m_group_id = '';
                        
                        $query = "SELECT DISTINCT
                                        vips_gruppe.Gruppenid
                                        , vips_gruppe.Gruppenname
                                FROM
                                        vips_gruppe
                                        , vips_inGruppe
                                WHERE
                                        vips_gruppe.Kursid = '$m_course_id'
                                        AND vips_inGruppe.user_id = '$m_user_id'
                                        AND vips_inGruppe.group_id = vips_gruppe.Gruppenid
                                        AND vips_inGruppe.end IS NULL
                                ";   //cdeiwiks: ... IS NULL ist wichtig: bedeutet: Gruppe ist aktuell und Mitgliedschaft noch nicht beendet
                        
                        $result = $vips_db->executeQuery($query);
                        
                        if ($result->next())
                        {
                                $m_group_id = $result->get("Gruppenid");
                                $m_group_name = $result->get("Gruppenname");
                        }
                        
                        if ($m_group_id == '')
                        {
                                $m_solvers[] = array(
                                        "id"            => $m_user_id,
                                        "name"          => $fullname,
                                        "username"      => $username,
                                        "single_solver" => true);
                        }
                        else
                        {
                            if (!is_array($m_groups[$m_group_name]))
                            {
                                    // $m_group_name as key is okay, as group name is unique
                                    $m_groups[$m_group_name] = array(
                                            "id"              => $m_group_id,
                                            "name"            => $m_group_name,
                                            "single_solver"   => false
                                            );
                            }

                            $m_groups[$m_group_name]['user'][] =
                                    array("name" => $fullname, "username" => $username);
                        }
                }
                
                ksort($m_groups);
                $m_solvers = array_merge($m_solvers, $m_groups);
        }
        
        //////////////////////////////////////////////////////////////////////
        // Fetch solution and points for each group/user for each exercise. //
        //////////////////////////////////////////////////////////////////////
        
        foreach ($m_solvers as $m_solver)
        {
                // look for solutions of this solver
                $m_solver_solutions = array();
                $query = "SELECT DISTINCT
                                $g_solution_table.`Aufgabenid` AS exerciseID
                        FROM
                                $g_solution_table
                        WHERE
                                Assignmentid = '$assignment'
                                AND ".($m_solver['single_solver'] ? 'Userid' : 'Gruppenid')." = '".$m_solver['id']."'
                                AND isLastSolution = 't'
                        ";
                $result = $vips_db->executeQuery($query);
                
                while ($result->next())
                {
                        $m_solver_solutions[$result->get("exerciseID")] = true;
                }
                
                // for all exercises in this assignment...
                foreach ($m_all_exercises as $m_exercise)
                {
                        if ($m_solver_solutions[$m_exercise['id']]) {
                                $students[] = $m_solver;
                                break;
                        }
                }
        }
        
        $t_main = $vipsTemplateFactory->open('export_student_overview');
        
        $t_main->set_attribute("type", $type);
        $t_main->set_attribute("assignment", $assignment);
        $t_main->set_attribute("name", $name);
        $t_main->set_attribute("students", $students);
        $t_main->set_attribute("mode", $mode);
        $t_main->set_attribute("print_correction_default", $print_correction_default);
        $t_main->set_attribute("print_sample_solution_default", $print_sample_solution_default);
        
        $t_main->set_attributes(array(
                'buttonpath'     => $g_outlet->buttonpath,
                'picturepath'    => $g_outlet->picturepath,
        'vips_imagepath' => $vipsPlugin->getPluginPath().'/images/',
                'status'             => $g_outlet->status));
        
        echo $t_main->render();
}







/**
 * print solution of one student for a given assignment and exercise id
 */
function print_student_solution ($exercise, $samplesolution = FALSE) {
        global $auth, $range_id, $perm, $vips_db, $vipsTemplateFactory;
        
        // Database variables
        global $g_inAssignment_table, $g_solution_table, $g_solution_points_table,
                $g_assignment_table, $g_outlet;
        
        $type             = getExternalParam("type");
        $assignment       = getExternalParam("assignment");
        $user             = getExternalParam("user");
        $single           = getExternalParam("single");
        $print_correction = getExternalParam("print_correction");
        
        if ($type == "exams") {
                $g_assignment_table     = 'vips_klausur';
                $g_inAssignment_table   = 'vips_inKlausur';
                $g_solution_table       = 'vips_K_Loesung';
                $g_solution_points_table= 'vips_K_Loesung_Punkte';
        } else {
                $g_assignment_table     = 'vips_uebungsblatt';
                $g_inAssignment_table   = 'vips_inUebungsblatt';
                $g_solution_table       = 'vips_T_Loesung';
                $g_solution_points_table= 'vips_T_Loesung_Punkte';
        }
        
        /*************************************************************************/
        /* Database: get exercise data for given exercise
        /*************************************************************************/
        
        $query = "SELECT vips_aufgabe.Aufgabe
                        , vips_aufgabe.URI
                        , vips_aufgabe.Name
                        , $g_inAssignment_table.Punkte
                        , $g_inAssignment_table.Position
                FROM vips_aufgabe
                        , $g_inAssignment_table
                WHERE vips_aufgabe.ID = '$exercise'
                        AND $g_inAssignment_table.vips_aufgabe = vips_aufgabe.ID
                        AND $g_inAssignment_table.$g_assignment_table = '$assignment'
                ";
        $result = $vips_db->executeQuery($query);
        if ($result->next())
        {
                $m_exercise_type       = $result->get("URI");
                $m_exercise_name       = $result->get("Name");
                $m_exercise_position   = $result->get("Position");
        }
        else
        {
            // FIXME generate error message here (no exercise data)
        }
        
        /*************************************************************************/
        /* Database: get solution data for given exercise and given group/user
        /*************************************************************************/
        
        if ($type == "sheets" && !$single)
        {
                $query = "SELECT Loesungsid, Loesung, visibleForStudent
                        FROM
                                ".$g_solution_table."
                        WHERE
                                Aufgabenid='$exercise' AND
                                Gruppenid = '$user' AND
                                Assignmentid='$assignment' AND
                                isLastSolution='t'";
        }
    else if ($type == "exams" || $single)
    {
            $query = "SELECT Loesungsid, Loesung, visibleForStudent
                                    FROM
                                            ".$g_solution_table."
                                    WHERE
                                            Aufgabenid='$exercise' AND
                                            Userid='$user' AND
                                            Assignmentid='$assignment' AND
                                            isLastSolution='t'";
    }

    $result = $vips_db->executeQuery($query);
        
        if ($result->next()) {
                $m_exercise_solution_id = $result->get("Loesungsid");
                $m_visible_for_student  = $result->get("visibleForStudent");
        }
        
        ###############################################################################
        # cdeiwiks vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
        #
        
        $query = "SELECT Punkte
                FROM $g_inAssignment_table
                WHERE $g_assignment_table = '$assignment'
                        AND vips_aufgabe = '$exercise'
                ";
        $result = $vips_db->executeQuery($query);
        if ($result->next())
        {
                $entry = $result->getRow();
                $gesamtpunktzahl = $entry['Punkte'];
        }
        
    #
    # cdeiwiks ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    ###################################################################################

        $query = "SELECT DISTINCT
                    Korrektur AS LecturerComment
                    , $g_solution_points_table.Punkte AS ReachedPoints
                    , $g_inAssignment_table.Punkte AS MaxPoints
            FROM
                    $g_inAssignment_table
                    , $g_solution_table
                    , $g_solution_points_table
            WHERE
                    $g_assignment_table = $assignment
                    AND vips_aufgabe = $exercise
                    AND Position = $m_exercise_position
                    AND Assignmentid = $assignment
                    AND Aufgabenid = $exercise
                    AND isLastSolution = 't'
                    AND idLoesung = Loesungsid
                    AND ".($type == "exams" || $single ? "Userid" : "Gruppenid")." = '$user'
                    ";
    $result = $vips_db->executeQuery($query);

    if ($result->next()) {
            $m_reached_points = $result->get("ReachedPoints");
            $m_max_points = $result->get("MaxPoints");
            $m_lecturer_comment = $result->get("LecturerComment");
    } else {
            // falls noch keine korrektur durchgefuehrt wurde
            $m_max_points = $gesamtpunktzahl;
    }
        
        // Unset $_POST variables so that rnd variables (which are not set for every
        // exercise) are deleted. If you do not do this, errors will occur as eg
        // $_POST["in_rnd_exercise"] won't ever become 0 again after a rnd exercise
        // in the assignment...
        unset($_POST['in_rnd_exercise']);
        unset($_POST['rnd_exercise_id']);
        unset($_POST['rnd_exercise_name']);
        unset($_POST['rnd_exercise_position']);
        
        if ($m_exercise_type == "rnd_exercise")
        {
                $_POST["in_rnd_exercise"]   = 1;
                $_POST["rnd_exercise_id"]       = $exercise;
                $_POST["rnd_exercise_name"] = htmlentities($m_exercise_name);
                $_POST["rnd_exercise_position"] = $m_exercise_position;
        }
        
        $_POST["exercise_id"]       = $exercise;
        $_POST['submitted']         = $submitted;
        $_POST['korrektur']         = 1;
        $_POST['exercise_position'] = $m_exercise_position;
        $_POST['print']             = 1;
        $_POST["samplesolution"]    = $samplesolution;
#       $_POST["assignment_type"]   = $type;  // needed in vips_assignments (?needed any more?)
        $_POST['assignment_id']     = $assignment;  // needed in vips_assignments
        $_POST["solver_id"]         = $user;  // needed in vips_assignments
        $_POST["single_solver"]     = $single;  // needed in vips_assignments
        $_POST["print_correction"]  = $print_correction;
        
        vips_assignments($type, "show_exercise");
        
   // template for showing lecturer's correction form
   $tmpl = $vipsTemplateFactory->open("solutions_edit_solution");
        
   $tmpl->set_attribute("t_reached_points", $m_reached_points);
   $tmpl->set_attribute("t_max_points", $m_max_points);
   $tmpl->set_attribute("t_lecturer_comment", $m_lecturer_comment);
   $tmpl->set_attribute("print", true);
   $tmpl->set_attribute("print_correction", $print_correction);
        
   echo $tmpl->render();
   
   return array('reached_points' => $m_reached_points, 'max_points' => $m_max_points);
}

/**
 * Discard any output buffered since last ob_start()
 */
function discard_buffer ($buffer)
{
    return '';
}

/**
 * Creates html print view of a sheet/exam (new window) specified by id
 * 
 * type = [sheets|exams]
 * mode = [blank|onlysample|student|samplesolution]   //blanko|blanko mit Musterloesung|studentenloesung ohne musterloesung|mit musterloesung]
 * assignment = sheet/exam id
 */
function print_assignment ($print_html_head = true) {
    global $vips_db, $g_outlet, $vipsTemplateFactory;
    global $vipsPlugin, $_include_additional_header;
    
    $type                  = getExternalParam("type");
    $assignment            = getExternalParam("assignment");
    $user                  = getExternalParam("user");
    $single                = getExternalParam("single");
    $print_correction      = getExternalParam("print_correction");
    $print_sample_solution = getExternalParam("print_sample_solution");
    
    $config = new Config();
    $stud_id = $config->getValue('VIPS_STUDENT_ID_DATAFIELD');
    
    if ($type == 'sheets') {
        $g_assignment_table = 'vips_uebungsblatt';
        $g_inAssignment_table = 'vips_inUebungsblatt';
    }
    else {
        $g_assignment_table = 'vips_klausur';
        $g_inAssignment_table = 'vips_inKlausur';
    }
    
    #######################
    # get assignment data #
    #######################
    
    $query = "SELECT * FROM $g_assignment_table WHERE ID = '$assignment'";
    $result = $vips_db->executeQuery($query);
    $result->next();
    $entry = $result->getRow();
    $assignment_name  = $entry['Name'];
    $assignment_start = $entry['Beginn'];
    $assignment_end   = $entry['Ende'];
    $printable        = $entry['Sichtbarkeit'];
    $show_corrections = $entry['korrekturenSichtbar'] == 't';
    
    $query = "SELECT VeranstaltungsNummer FROM seminare WHERE Seminar_id = '$g_outlet->courseID'";
    $result = $vips_db->executeQuery($query);
    $result->next();
    $entry = $result->getRow();
    $course_number = $entry['VeranstaltungsNummer'];
    
    if ($g_outlet->status == 'student')
    {
        /* get group of student */
        $result = $vips_db->executeQuery("SELECT vips_inGruppe.group_id
            FROM vips_inGruppe, vips_gruppe
            WHERE vips_inGruppe.user_id = '$g_outlet->userID' AND
                vips_gruppe.Gruppenid = vips_inGruppe.group_id AND
                vips_gruppe.Kursid = '$g_outlet->courseID' AND
                vips_inGruppe.end IS NULL");
        if ($result->next())
        {
            $group_id = $result->get('group_id');
        }
        
        if (!$printable)
        {
            echo _('Zugriff nur fr Dozenten erlaubt.');
            return;
        }
        else if ($user && ($single && $g_outlet->userID != $user || !$single && $group_id != $user))
        {
            echo _('Zugriff nur auf eigene Klausuren erlaubt.');
            return;
        }
        else if ($print_sample_solution && !$show_corrections)
        {
            echo _('Zugriff erst nach Freischaltung durch den Dozenten mglich.');
            return;
        }
    }
    
    $t_main_header = $vipsTemplateFactory->open('print_assignment_header');
    $t_main_header->set_attributes(array(
        'buttonpath'  => $g_outlet->buttonpath,
        'picturepath' => $g_outlet->picturepath,
        'status'      => $g_outlet->status));
    
    if ($print_html_head) {
        // this hack is necessary to disable the standard Stud.IP layout
        ob_end_clean();
        
        $_include_additional_header =
            '<link rel="stylesheet" href="'.$vipsPlugin->getPluginPath().'/css/vips_print.css" type="text/css">';
        // note: second include of 'html_head.inc.php' fails on Stud.IP 1.2-os1
        if (function_exists('html_head')) {
            echo html_head();
        } else {
            include 'html_head.inc.php';
        }
    }
    
    ################
    # get semester #
    ################
    
    $result = $vips_db->executeQuery("SELECT semester_data.name FROM semester_data, seminare
        WHERE seminare.Seminar_id = '$g_outlet->courseID'
            AND semester_data.beginn = seminare.start_time");
    $result->next();
    $entry = $result->getRow();
    $semester = $entry['name'];    //z.B. WS 2005
    
    #################
    # get lecturers #
    #################
    
    $lecturers = array();
    
    $sql = "SELECT auth_user_md5.* 
        FROM auth_user_md5, seminar_user
        WHERE seminar_user.seminar_id = '$g_outlet->courseID'
            AND auth_user_md5.user_id = seminar_user.user_id
            AND seminar_user.status = 'dozent'";
    $result = $vips_db->executeQuery($sql);
    
    while ($result->next()) {
        $lecturer = $result->getRow();
        $lecturers[] = $lecturer;
    }
    
    ####################
    # get participants #
    ####################
    
    if ($user)
    {
        if ($type == "sheets" && !$single)
        {
            // sort participants by last name, first name
            $query = "SELECT vips_inGruppe.user_id
                FROM vips_inGruppe,
                    auth_user_md5
                WHERE group_id = $user
                    AND vips_inGruppe.end IS NULL
                    AND vips_inGruppe.user_id = auth_user_md5.user_id
                ORDER BY Nachname, Vorname";
            $result = $vips_db->executeQuery($query);
            
            while ($result->next())
            {
                $user_id = $result->get('user_id');
                $students[] = get_fullname($user_id);
                $fields = new DataFields($user_id);
                $fields = $fields->getLocalFields();
                $stud_ids[] = $fields[$stud_id]['content'];
                // DataFieldEntry is not yet available
                // $fields = DataFieldEntry::getDataFieldEntries($user_id);
                // $stud_ids[] = $fields[$stud_id]->getValue();
            }
        }
        else
        {
            $students[] = get_fullname($user);
            $fields = new DataFields($user);
            $fields = $fields->getLocalFields();
            $stud_ids[] = $fields[$stud_id]['content'];
            // DataFieldEntry is not yet available
            // $fields = DataFieldEntry::getDataFieldEntries($user);
            // $stud_ids[] = $fields[$stud_id]->getValue();
        }
    }
    
    $t_main_header->set_attribute("course_name",$g_outlet->course);
    $t_main_header->set_attribute("course_number",$course_number);
    $t_main_header->set_attribute("lecturers",$lecturers);
    $t_main_header->set_attribute("semester",$semester);
    $t_main_header->set_attribute("students",$students);
    $t_main_header->set_attribute("stud_ids",$stud_ids);
    
    $t_main_header->set_attribute("assignment_name",$assignment_name);
    $t_main_header->set_attribute("assignment_start",$assignment_start);
    $t_main_header->set_attribute("assignment_end",$assignment_end);
    
    echo $t_main_header->render();
    
    if (!$user)  // without student solution
    {
        
        #####################
        # get all exercises #
        #####################
        
        $query = "SELECT vips_aufgabe.*,
                vips_aufgaben_typen.Typenname,
                $g_inAssignment_table.Position,
                $g_inAssignment_table.Punkte
            FROM vips_aufgabe,
                vips_aufgaben_typen,
                $g_inAssignment_table
            WHERE $g_assignment_table = '$assignment'
                AND vips_aufgabe = vips_aufgabe.ID
                AND vips_aufgaben_typen.URI = vips_aufgabe.URI
            ORDER BY $g_inAssignment_table.Position
            ";
        $result = $vips_db->executeQuery($query);
        $exercises = array();
        while ($result->next())
        {
            $res = $result->getRow();
            array_push($exercises, $res);
        }
        
        // process each exercise in the assignment
        while ($exercise = array_shift($exercises))
        {
            $exercise_id       = $exercise["ID"];
            $exercise_type     = $exercise['URI'];
            $exercise_title    = $exercise['Name'];
            $exercise_xml      = $exercise['Aufgabe'];
            $exercise_position = $exercise['Position'];
            $max_points        = $exercise["Punkte"];
            
            // by default, show the given exercise body (which is one in number)
            $number_of_exercises_to_show = 1;
            
            // BUT: random exercise contains several subexercises -- show them all!
            if ($exercise_type == "rnd_exercise")
            {
                require_once("exercises/rnd_exercise.php");
                $rnd0_exercise = new rnd_exercise();
                // Attention: case sensitive!
                $subexercise_ids = $rnd0_exercise->getAttributesFromXML($exercise_xml, "SubExercise", "ID");
                // insert id of random exercise to the beginning of the array: this
                // will allow rnd-exercise to print something itself before all the
                // other subexercises are processed
                array_unshift($subexercise_ids, $exercise_id);
                $number_of_exercises_to_show = count($subexercise_ids);
                
                if ($number_of_exercises_to_show == 1)  // no subexercises!
                {
                    $rnd_exercise_empty = 1;
                }
                else
                {
                    $rnd_exercise_empty = 0;
                }
            }
            
            // now, show the exercise body/bodies
            for ($i = 0; $i < $number_of_exercises_to_show; $i++)
            {
                if ($exercise_type == "rnd_exercise")
                {
                    // backup of rnd exercise info
                    $in_rnd_exercise       = 1;
                    $rnd_exercise_id       = $exercise_id;
                    $rnd_exercise_title    = $exercise_title;
                    $rnd_exercise_xml      = $exercise_xml;
                    $rnd_exercise_position = $exercise_position;
                    
                    $subexercise_id = array_shift($subexercise_ids);
                    
                    // fetch info about the current SUBexercise
                    $query = "SELECT DISTINCT
                            ID
                            , URI
                            , Name
                            , Aufgabe
                        FROM
                            vips_aufgabe
                        WHERE
                            ID = '".$subexercise_id."'
                        ";
                    $result = $vips_db->executeQuery($query);
                    $result->next();
                    
                    // store info about the subexercise
                    $exercise_id    = $result->get("ID");
                    $exercise_type  = $result->get("URI");
                    $exercise_title = $result->get("Name");
                    $exercise_xml   = $result->get("Aufgabe");
                }
                
                //////////////////////////// INFO ////////////////////////////
                // If we are in a random exercise, the $exercise_...        //
                // variables now contain information about the SUBexercise. //
                // Otherwise (default), these variables just describe the   //
                // current exercise.                                        //
                //////////////////////////////////////////////////////////////
                
                if ($exercise_type == 'zu_exercise')
                {}
                else
                {
                    require_once("exercises/$exercise_type.php");
                    
                    $o_exercise = new $exercise_type();
                    
                    $exercise_question      = $o_exercise->getTagsFromXML($exercise_xml, "Question");
                    $exercise_question  = $exercise_question[0];
                    $exercise_hint          = $o_exercise->getTagsFromXML($exercise_xml, "Hint");
                    $exercise_hint          = $exercise_hint[0];
                    //wikiformat!
                    $exercise_question      = formatReady($exercise_question);
                    $exercise_hint          = formatReady($exercise_hint);
                    
                    $dom_xml = $o_exercise->getDomXML($exercise_xml);
                    $answers = $dom_xml->get_elements_by_tagname("Answer");
                    
                    $answer_array   = array();
                    $correct_array  = array();
                    $answer_default = '';
                    
                    if ($exercise_type == 'sc_exercise' || $exercise_type == 'sco_exercise')
                    {
                        foreach ($answers as $key=>$node) {
                            $group = $node->get_attribute("group");
                            if ($group == '') $group = 0;
                            
                            $answer_array[$group][] = formatReady(utf8_decode($node->get_content()));
                            if ($node->get_attribute("Correct") == '1')
                                $correct_array[$group] = count($answer_array[$group]) - 1;
                        }
                    }
                    elseif ($exercise_type == 'yn_exercise')
                    {
                        foreach ($answers as $key=>$node) {
                            $answer_array[]         = formatReady(utf8_decode($node->get_content()));
                            if ($node->get_attribute("Correct") == '1')
                                $correct_array[0]       = $key;
                        }
                    }
                    elseif ($exercise_type == 'mc_exercise'||
                        $exercise_type == 'mco_exercise'||
                        $exercise_type == 'lt_exercise')
                    {
                        foreach ($answers as $node) {
                            $answer_array[]         = formatReady(utf8_decode($node->get_content()));
                            $correct_array[]        = $node->get_attribute("Correct");
                            //$mc_auswertung                = $o_exercise->getTagsFromXML($exercise_xml, "EvalMode");
                            //$mc_auswertung                = $mc_auswertung[0];
                        }
                    }
                    elseif ($exercise_type == 'tb_exercise'||
                        $exercise_type == 'pl_exercise'||
                        $exercise_type == 'oc_exercise'||
                        $exercise_type == 'oct_exercise')
                    {
                        $node = $answers[0];
                        $answer_array[0]        = html_ready(utf8_decode($node->get_content()));
                        
                        if ($exercise_type == 'tb_exercise') {
                            $answer_default         = $o_exercise->getTagsFromXML($exercise_xml, "AnswerDefault");
                            $answer_default         = html_ready($answer_default[0]);
                        }
                        elseif ($exercise_type == 'pl_exercise') {
                            $answer_default         = $o_exercise->getTagsFromXML($exercise_xml, "AnswerDefault");
                            $answer_default         = html_ready($answer_default[0]);
                            $query_default          = $o_exercise->getTagsFromXML($exercise_xml, "QueryDefault");
                            $query_default          = html_ready($query_default[0]);
                        }
                        else if ($exercise_type == "oc_exercise") {
                            $answer_default = $o_exercise->getTagsFromXML($exercise_xml, "Answer");
                            $answer_default = html_ready($answer_default[0]);
                        }
                    }
                    else if ($exercise_type == "rh_exercise")
                    {
                        $defaults = $dom_xml->get_elements_by_tagname("Default");
                        $default_array = array();
                        
                        foreach ($defaults as $node)
                        {
                            $default_array[] = formatReady(utf8_decode($node->get_content()));
                        }
                        
                        foreach ($answers as $node)
                        {
                            $answer_array[] = formatReady(utf8_decode($node->get_content()));
                            $correct_array[] = $node->get_attribute("Default");
                        }
                    }
                    elseif ($exercise_type == 'zu_exercise')
                    {
                        $exercise_picurl = $o_exercise->getTagsFromXML($exercise_xml, "PictureURL");
                        $exercise_picurl = urldecode($exercise_picurl[0]);
                        foreach ($answers as $node)
                        {
                            $solution = array();
                            $solution[0] = $node->get_attribute("x1"); 
                            $solution[1] = $node->get_attribute("y1"); 
                            $solution[2] = $node->get_attribute("x2"); 
                            $solution[3] = $node->get_attribute("y2"); 
                            $answer_array[]         = urldecode(utf8_decode($node->get_content()));
                            $correct_array[]        = $solution;
                        }
                    }
                }
                
                // if 'for' loop runs for the first time: echo exercise title
                if ($i == 0)
                {
                    echo '<div style="page-break-inside: avoid">';
                    echo '<table border="0" width="99%">';
                    echo '<tr><td colspan="2">';
                    echo '<table style="width: 100%;"><tr><td>';
                    echo '<b><u>'.$exercise_position.". ".$exercise_title.'</u></b>';
                    echo '</td><td style="text-align: right; white-space: nowrap;">';
                    echo sprintf(ngettext('(%s Punkt)', '(%s Punkte)', $max_points), $max_points);
                    echo "</td></tr></table>";
                    echo '</td></tr>';
                }
                
                // for every exercise within the 'for' loop: echo exercise question
                echo "<tr><td colspan=\"2\"><p>";
                if ($i > 0)  // if there are any subexercises, show numbering before each one
                {
                    echo chr(ord($i)+48).")&nbsp;";  // changes "1" in "a", "2" in "b", ...
                }
                echo $exercise_question.'</p>';
                //if ($exercise_hint) {}
                echo '</td></tr>';
                
                $t_main = $vipsTemplateFactory->open("exercises/show_$exercise_type");
                $t_main->set_attribute("exercise",$exercise);
                $t_main->set_attribute("answer_default", $answer_default);
                $t_main->set_attribute("default_array", $default_array);  // for rh-exercise
                $t_main->set_attribute("answer_array", $answer_array);
                $t_main->set_attribute("correct_array",$correct_array);
                //$t_main->set_attribute("student_array",$student_array);
                $t_main->set_attribute("rnd_exercise_empty", $rnd_exercise_empty);
                
                $t_main->set_attribute("print", true);
                $t_main->set_attribute("samplesolution", $print_sample_solution);
                $t_main->set_attribute("pluginpath", $vipsPlugin->getPluginPath());
                
                if ($exercise_type == 'zu_exercise')
                {
                    echo '<tr><td colspan="2">--- Keine Druckansicht m&ouml;glich ---</td></tr><tr><td>&nbsp;</td></tr>';
                }
                else
                {
                    echo $t_main->render();
                }
                
                // restore rnd_exercise_data
                if ($in_rnd_exercise)
                {
                    $in_rnd_exercise = 0;
                    $exercise_id    = $rnd_exercise_id;
                    $exercise_type  = "rnd_exercise";
                    $exercise_title = $rnd_exercise_title;
                    $exercise_xml   = $rnd_exercise_xml;
                }
            }  // end for : for each SUBexercise (default: just one!)
        }  // end while : for each exercise
        echo '</table>';
        echo '</div>';
    }  // end if : without student solution
    
    else  // if ($user) = with student solution
    {
        
        #####################
        # get all exercises #
        #####################
        
        $query = "SELECT vips_aufgabe.*,
                vips_aufgaben_typen.Typenname,
                $g_inAssignment_table.Position,
                $g_inAssignment_table.Punkte
            FROM vips_aufgabe,
                vips_aufgaben_typen,
                $g_inAssignment_table
            WHERE $g_assignment_table = '$assignment'
                AND vips_aufgabe = vips_aufgabe.ID
                AND vips_aufgaben_typen.URI = vips_aufgabe.URI
            ORDER BY $g_inAssignment_table.Position
            ";
        $result = $vips_db->executeQuery($query);
        
        while ($result->next()) {
            $exercise = $result->getRow();
            $exercises[] = $exercise['ID'];
        }
        $reached_points = 0;
        $max_points = 0;

        foreach ($exercises as $exercise) {
            $points_array = print_student_solution($exercise, $print_sample_solution);
            $reached_points += $points_array['reached_points'];
            $max_points     += $points_array['max_points'];
        }

        if ($print_correction)
        {
            $tmpl = $vipsTemplateFactory->open('export_print_footer');
            $tmpl->set_attribute('reached_points', $reached_points);
            $tmpl->set_attribute('max_points', $max_points);
            echo $tmpl->render();
        }
    }  // end if : with student solution

    if ($print_html_head) {
        // this hack is necessary to disable the standard Stud.IP layout
        include 'html_end.inc.php';
        ob_start('discard_buffer');
    }
}

/**
 * Creates html print view of a sheet/exam (new window) specified by id
 * This function just calls print_assignment() for each user or group in turn.
 * 
 * type = [sheets|exams]
 * mode = [student|samplesolution]   //[ohne musterloesung|mit musterloesung]
 * assignment = sheet/exam id
 */
function print_all_assignments ()
{
    global $vipsPlugin, $_include_additional_header;

    $user = getExternalParam("user");
    $single = getExternalParam("single");

    // this hack is necessary to disable the standard Stud.IP layout
    ob_end_clean();

    $_include_additional_header =
        '<link rel="stylesheet" href="'.$vipsPlugin->getPluginPath().'/css/vips_print.css" type="text/css">';
    // note: second include of 'html_head.inc.php' fails on Stud.IP 1.2-os1
    if (function_exists('html_head')) {
        echo html_head();
    } else {
        include 'html_head.inc.php';
    }

    // when printing for all students, $user is an array
    for ($i = 0; $i < count($user); ++$i)
    {
        $_POST['user'] = $user[$i];
        $_POST['single'] = $single[$i];

        print_assignment(false);
        echo '<p style="page-break-after: always;">';
    }

    // this hack is necessary to disable the standard Stud.IP layout
    include 'html_end.inc.php';
    ob_start('discard_buffer');
}

/**
 * 
 * Displays overview of all assignments
 * 
 */
function print_overview() {
        global $vips_db, $g_outlet, $vipsTemplateFactory, $vipsPlugin;

        $t_main = $vipsTemplateFactory->open('export_print_overview');
        $sheets = array();
        $exams = array();

        $query = "SELECT * FROM vips_uebungsblatt
                        WHERE
                                Kurs = '$g_outlet->courseID'
                        ORDER BY
                                Beginn, Position";

        $result = $vips_db->executeQuery($query);

        while ($result->next()) {
                $entry = $result->getRow();
                if ($g_outlet->status != 'student') {
                        $entry['korrekturenSichtbar'] = 't';
                }
                if ($g_outlet->status != 'student' || $entry['Sichtbarkeit']) {
                        $sheets[] = $entry;
                }
        }
        
        $query = "SELECT * FROM vips_klausur
                        WHERE
                                Kurs='$g_outlet->courseID'
                        ORDER BY
                                Beginn, Position";

        $result = $vips_db->executeQuery($query);

        while ($result->next()) {
                $entry = $result->getRow();
                if ($g_outlet->status != 'student') {
                        $entry['korrekturenSichtbar'] = 't';
                }
                if ($g_outlet->status != 'student' || $entry['Sichtbarkeit']) {
                        $exams[] = $entry;
                }
        }
        
        $query = "SELECT `group_id`
                FROM `vips_inGruppe`
                WHERE `user_id` = '".$g_outlet->userID."'
                        AND `end` IS NOT NULL
                ";
        $result = $vips_db->executeQuery($query);
        if ($result->next())
        {
                $entry = $result->getRow();
                $user = $entry["group_id"];
                $single = 0;
        }
        else
        {
                $user = $g_outlet->userID;
                $single = 1;
        }
        
        $t_main->set_attribute("sheets",$sheets);
        $t_main->set_attribute("exams",$exams);
        
        $t_main->set_attributes(array(
                "user"           => $user,
                "single"         => $single,
                'buttonpath'     => $g_outlet->buttonpath,
                'picturepath'    => $g_outlet->picturepath,
                'vips_imagepath' => $vipsPlugin->getPluginPath().'/images/',
                'status'         => $g_outlet->status));

        echo $t_main->render();
}

?>
